﻿using System;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using ThinkGeo.MapSuite.Core;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.IO;
using System.Text;
using System.Collections.ObjectModel;

public partial class _Default : System.Web.UI.Page, ICallbackEventHandler
{
    private const double bufferFactor = 10;
    private string callbackResult = String.Empty;

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 233, 232, 214));
            Map1.CurrentExtent = new RectangleShape(-97.766, 30.291, -97.755, 30.286);
            Map1.MapUnit = GeographyUnit.DecimalDegree;

            ShapeFileFeatureLayer roadLayer = new ShapeFileFeatureLayer(Server.MapPath(@"App_Data\austinstreets.shp"));
            roadLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.LocalRoad1;
            roadLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = TextStyles.MajorStreet2("FENAME");
            roadLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            roadLayer.DrawingMarginPercentage = 100;

            Map1.StaticOverlay.Layers.Add("RoadLayer", roadLayer);
        }
    }

    #region ICallbackEventHandler Members

    public string GetCallbackResult()
    {
        return callbackResult;
    }

    public void RaiseCallbackEvent(string eventArgument)
    {
        ShapeFileFeatureLayer roadLayer = (ShapeFileFeatureLayer)Map1.StaticOverlay.Layers["RoadLayer"];

        DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(ClickInformation));
        ClickInformation clickInfo = (ClickInformation)serializer.ReadObject(new MemoryStream(Encoding.Default.GetBytes(eventArgument)));
        RectangleShape extent = clickInfo.ReadExtent();
        Map1.CurrentExtent = extent;
        PointShape clickScreenPoint = clickInfo.ReadPointShape();

        PointShape clickWorldPoint = Map1.ToWorldCoordinate((float)clickScreenPoint.X, (float)clickScreenPoint.Y);
        PointShape edgeScreenPoint = new PointShape(clickScreenPoint.X + bufferFactor, clickScreenPoint.Y);
        PointShape edgeWorldPoint = Map1.ToWorldCoordinate((float)edgeScreenPoint.X, (float)edgeScreenPoint.Y);
        double distance = ExtentHelper.GetWorldDistanceBetweenTwoScreenPoints(Map1.CurrentExtent, (float)clickScreenPoint.X, (float)clickScreenPoint.Y, (float)edgeScreenPoint.X, (float)edgeScreenPoint.Y, (float)Map1.Width.Value, (float)Map1.Height.Value, GeographyUnit.DecimalDegree, DistanceUnit.Meter);

        MultipolygonShape bufferedShape = clickWorldPoint.Buffer(distance, GeographyUnit.DecimalDegree, DistanceUnit.Meter);

        roadLayer.Open();
        Collection<Feature> features = roadLayer.QueryTools.GetFeaturesIntersecting(bufferedShape, ReturningColumnsType.AllColumns);
        roadLayer.Close();

        if (features.Count != 0)
        {
            Feature feature = features[0];
            string feName = feature.ColumnValues["FENAME"];
            callbackResult = feName;
        }
        else
        {
            callbackResult = String.Empty;
        }
    }
    #endregion
}

[DataContract]
public class ClickInformation
{
    [DataMember(Name = "x")]
    public double X { get; set; }

    [DataMember(Name = "y")]
    public double Y { get; set; }

    [DataMember(Name = "l")]
    public double Left { get; set; }

    [DataMember(Name = "t")]
    public double Top { get; set; }

    [DataMember(Name = "r")]
    public double Right { get; set; }

    [DataMember(Name = "b")]
    public double Bottom { get; set; }

    public PointShape ReadPointShape()
    {
        return new PointShape(X, Y);
    }

    public RectangleShape ReadExtent()
    {
        return new RectangleShape(Left, Top, Right, Bottom);
    }
}
